from enum import Enum


class Taxon(int, Enum):
    ARABIDOPSIS_THALIANA = 3702
    BOS_TAURUS = 9913
    CAENORHABDITIS_ELEGANS = 6239
    CHLAMYDOMONAS_REINHARDTII = 3055
    DANIO_RERIO = 7955
    DICTYOSTELIUM_DISCOIDEUM = 44689
    DROSOPHILA_MELANOGASTER = 7227
    ESCHERICHIA_COLI = 562
    HEPACIVIRUS_HOMINIS = 3052230
    HOMO_SAPIENS = 9606
    MUS_MUSCULUS = 10090
    MYCOPLASMOIDES_PNEUMONIAE = 2104
    ORYZA_SATIVA = 4530
    PLASMODIUM_FALCIPARUM = 5833
    PNEUMOCYSTIS_CARINII = 4754
    RATTUS_NORVEGICUS = 10116
    SACCHAROMYCES_CEREVISIAE = 4932
    SCHIZOSACCHAROMYCES_POMBE = 4896
    TAKIFUGU_RUBRIPES = 31033
    XENOPUS_LAEVIS = 8355
    ZEA_MAYS = 4577


class Input(str, Enum):
    """
    These are valid input database types for the BioDBNet API.
    """

    AFFY_GENECHIP_ARRAY = "Affy GeneChip Array"
    AFFY_ID = "Affy ID"
    AFFY_TRANSCRIPT_CLUSTER_ID = "Affy Transcript Cluster ID"
    AGILENT_ID = "Agilent ID"
    BIOCARTA_PATHWAY_NAME = "Biocarta Pathway Name"
    CODELINK_ID = "CodeLink ID"
    DBSNP_ID = "dbSNP ID"
    DRUGBANK_DRUG_ID = "DrugBank Drug ID"
    DRUGBANK_DRUG_NAME = "DrugBank Drug Name"
    EC_NUMBER = "EC Number"
    ENSEMBL_GENE_ID = "Ensembl Gene ID"
    ENSEMBL_PROTEIN_ID = "Ensembl Protein ID"
    ENSEMBL_TRANSCRIPT_ID = "Ensembl Transcript ID"
    EST_ACCESSION = "EST Accession"
    FLYBASE_GENE_ID = "FlyBase Gene ID"
    GENBANK_NUCLEOTIDE_ACCESSION = "GenBank Nucleotide Accession"
    GENBANK_PROTEIN_ACCESSION = "GenBank Protein Accession"
    GENE_ID = "Gene ID"
    GENE_SYMBOL = "Gene Symbol"
    GENE_SYMBOL_AND_SYNONYMS = "Gene Symbol and Synonyms"
    GENE_SYMBOL_ORDERED_LOCUS = "Gene Symbol Ordered Locus"
    GENE_SYMBOL_ORF = "Gene Symbol ORF"
    GI_NUMBER = "GI Number"
    GO_ID = "GO ID"
    GSEA_STANDARD_NAME = "GSEA Standard Name"
    H_INV_LOCUS_ID = "H-Inv Locus ID"
    H_INV_PROTEIN_ID = "H-Inv Protein ID"
    H_INV_TRANSCRIPT_ID = "H-Inv Transcript ID"
    HGNC_ID = "HGNC ID"
    HMDB_METABOLITE = "HMDB Metabolite"
    HOMOLOGENE_ID = "HomoloGene ID"
    ILLUMINA_ID = "Illumina ID"
    INTERPRO_ID = "InterPro ID"
    IPI_ID = "IPI ID"
    KEGG_COMPOUND_ID = "KEGG Compound ID"
    KEGG_COMPOUND_NAME = "KEGG Compound Name"
    KEGG_DISEASE_ID = "KEGG Disease ID"
    KEGG_DRUG_ID = "KEGG Drug ID"
    KEGG_DRUG_NAME = "KEGG Drug Name"
    KEGG_GENE_ID = "KEGG Gene ID"
    KEGG_PATHWAY_ID = "KEGG Pathway ID"
    MAIZEGDB_ID = "MaizeGDB ID"
    MGI_ID = "MGI ID"
    MIM_ID = "MIM ID"
    MIRBASE_ID = "miRBase ID"
    MIRBASE_MATURE_MIRNA_ACC = "miRBase Mature miRNA Acc"
    NCIPID_PATHWAY_NAME = "NCIPID Pathway Name"
    ORGANISM_SCIENTIFIC_NAME = "Organism Scientific Name"
    PDB_ID = "PDB ID"
    PFAM_ID = "Pfam ID"
    PHARMGKB_ID = "PharmGKB ID"
    PUBCHEM_ID = "PubChem ID"
    REACTOME_PATHWAY_NAME = "Reactome Pathway Name"
    REFSEQ_GENOMIC_ACCESSION = "RefSeq Genomic Accession"
    REFSEQ_MRNA_ACCESSION = "RefSeq mRNA Accession"
    REFSEQ_PROTEIN_ACCESSION = "RefSeq Protein Accession"
    SGD_ID = "SGD ID"
    TAIR_ID = "TAIR ID"
    TAXON_ID = "Taxon ID"
    UNIGENE_ID = "UniGene ID"
    UNIPROT_ACCESSION = "UniProt Accession"
    UNIPROT_ENTRY_NAME = "UniProt Entry Name"
    UNIPROT_PROTEIN_NAME = "UniProt Protein Name"
    UNISTS_ID = "UniSTS ID"


class Output(str, Enum):
    """
    These are valid output database types for the BioDBNet API.
    """

    AFFY_ANNOTATION = "Affy Annotation"
    AFFY_ID = "Affy ID"
    AGILENT_ID = "Agilent ID"
    ALLERGOME_CODE = "Allergome Code"
    APIDB_CRYPTODB_ID = "ApiDB_CryptoDB ID"
    BIND_ID = "BIND ID"
    BIOCARTA_PATHWAY_DESCRIPTION = "Biocarta Pathway Description"
    BIOCARTA_PATHWAY_NAME = "Biocarta Pathway Name"
    BIOCARTA_PATHWAY_TITLE = "Biocarta Pathway Title"
    BIOCYC_ID = "BioCyc ID"
    CCDS_ID = "CCDS ID"
    CHEBI_ID = "ChEBI ID"
    CHROMOSOMAL_LOCATION = "Chromosomal Location"
    CLEANEX_ID = "CleanEx ID"
    CODELINK_ID = "CodeLink ID"
    COG_TERM = "COG Term"
    COSMIC_ID = "COSMIC ID"
    CPDB_PROTEIN_INTERACTOR = "CPDB Protein Interactor"
    CTD_DISEASE_INFO = "CTD Disease Info"
    CTD_DISEASE_NAME = "CTD Disease Name"
    CYGD_ID = "CYGD ID"
    DBSNP_ID = "dbSNP ID"
    DICTYBASE_ID = "dictyBase ID"
    DIP_ID = "DIP ID"
    DISPROT_ID = "DisProt ID"
    DPD_DIN = "DPD DIN"
    DRUGBANK_DRUG_ENZYME_ID = "DrugBank Drug Enzyme ID"
    DRUGBANK_DRUG_ID = "DrugBank Drug ID"
    DRUGBANK_DRUG_INFO = "DrugBank Drug Info"
    DRUGBANK_DRUG_NAME = "DrugBank Drug Name"
    DRUGBANK_DRUG_TARGET_ID = "DrugBank Drug Target ID"
    DRUGBANK_ID = "DrugBank ID"
    EC_NUMBER = "EC Number"
    ECHOBASE_ID = "EchoBASE ID"
    ECOGENE_ID = "EcoGene ID"
    ENSEMBL_BIOTYPE = "Ensembl Biotype"
    ENSEMBL_GENE_ID = "Ensembl Gene ID"
    ENSEMBL_GENE_INFO = "Ensembl Gene Info"
    ENSEMBL_PROTEIN_ID = "Ensembl Protein ID"
    ENSEMBL_TRANSCRIPT_ID = "Ensembl Transcript ID"
    EST_ACCESSION = "EST Accession"
    FLYBASE_GENE_ID = "FlyBase Gene ID"
    FLYBASE_PROTEIN_ID = "FlyBase Protein ID"
    FLYBASE_TRANSCRIPT_ID = "FlyBase Transcript ID"
    FUNCATDB_ID = "FunCatDB ID"
    GAD_DISEASE_INFO = "GAD Disease Info"
    GAD_DISEASE_NAME = "GAD Disease Name"
    GENBANK_NUCLEOTIDE_ACCESSION = "GenBank Nucleotide Accession"
    GENBANK_NUCLEOTIDE_GI = "GenBank Nucleotide GI"
    GENBANK_PROTEIN_ACCESSION = "GenBank Protein Accession"
    GENBANK_PROTEIN_GI = "GenBank Protein GI"
    GENE_ID = "Gene ID"
    GENE_INFO = "Gene Info"
    GENE_SYMBOL = "Gene Symbol"
    GENE_SYMBOL_AND_SYNONYMS = "Gene Symbol and Synonyms"
    GENE_SYMBOL_ORDERED_LOCUS = "Gene Symbol Ordered Locus"
    GENE_SYMBOL_ORF = "Gene Symbol ORF"
    GENE_SYNONYMS = "Gene Synonyms"
    GENEFARM_ID = "GeneFarm ID"
    GI_NUMBER = "GI Number"
    GO_BIOLOGICAL_PROCESS = "GO - Biological Process"
    GO_CELLULAR_COMPONENT = "GO - Cellular Component"
    GO_MOLECULAR_FUNCTION = "GO - Molecular Function"
    GO_ANNOTATION = "GO Annotation"
    GO_ID = "GO ID"
    GSEA_GROUP_MEMBERS = "GSEA Group Members"
    GSEA_STANDARD_NAME = "GSEA Standard Name"
    HINV_LOCUS_ID = "H-Inv Locus ID"
    HINV_PROTEIN_ID = "H-Inv Protein ID"
    HINV_TRANSCRIPT_ID = "H-Inv Transcript ID"
    HAMAP_ID = "HAMAP ID"
    HGNC_ID = "HGNC ID"
    HMDB_ENZYME_GENE_SYMBOL = "HMDB Enzyme - Gene Symbol"
    HMDB_ENZYME_UNIPROT_ENTRY_NAME = "HMDB Enzyme - UniProt Entry Name"
    HMDB_METABOLITE = "HMDB Metabolite"
    HMDB_METABOLITE_DESC = "HMDB Metabolite Desc"
    HMDB_METABOLITE_ID = "HMDB Metabolite ID"
    HOMOLOG_ALL_ENS_GENE_ID = "Homolog - All Ens Gene ID"
    HOMOLOG_ALL_ENS_PROTEIN_ID = "Homolog - All Ens Protein ID"
    HOMOLOG_ALL_GENE_ID = "Homolog - All Gene ID"
    HOMOLOG_HUMAN_ENS_GENE_ID = "Homolog - Human Ens Gene ID"
    HOMOLOG_HUMAN_ENS_PROTEIN_ID = "Homolog - Human Ens Protein ID"
    HOMOLOG_HUMAN_GENE_ID = "Homolog - Human Gene ID"
    HOMOLOG_MOUSE_ENS_GENE_ID = "Homolog - Mouse Ens Gene ID"
    HOMOLOG_MOUSE_ENS_PROTEIN_ID = "Homolog - Mouse Ens Protein ID"
    HOMOLOG_MOUSE_GENE_ID = "Homolog - Mouse Gene ID"
    HOMOLOG_RAT_ENS_GENE_ID = "Homolog - Rat Ens Gene ID"
    HOMOLOG_RAT_ENS_PROTEIN_ID = "Homolog - Rat Ens Protein ID"
    HOMOLOG_RAT_GENE_ID = "Homolog - Rat Gene ID"
    HOMOLOGENE_ID = "HomoloGene ID"
    HPA_ID = "HPA ID"
    HPRD_ID = "HPRD ID"
    HPRD_PROTEIN_COMPLEX = "HPRD Protein Complex"
    HPRD_PROTEIN_INTERACTOR = "HPRD Protein Interactor"
    ILLUMINA_ID = "Illumina ID"
    IMGT_GENEDB_ID = "IMGT/GENE-DB ID"
    INTACT_ID = "IntAct ID"
    INTERPRO_ID = "InterPro ID"
    IPI_ID = "IPI ID"
    KEGG_COMPOUND_ID = "KEGG Compound ID"
    KEGG_COMPOUND_NAME = "KEGG Compound Name"
    KEGG_DISEASE_ID = "KEGG Disease ID"
    KEGG_DISEASE_INFO = "KEGG Disease Info"
    KEGG_DRUG_ID = "KEGG Drug ID"
    KEGG_DRUG_INFO = "KEGG Drug Info"
    KEGG_DRUG_NAME = "KEGG Drug Name"
    KEGG_GENE_ID = "KEGG Gene ID"
    KEGG_ORTHOLOGY_ID = "KEGG Orthology ID"
    KEGG_PATHWAY_ID = "KEGG Pathway ID"
    KEGG_PATHWAY_INFO = "KEGG Pathway Info"
    KEGG_PATHWAY_TITLE = "KEGG Pathway Title"
    KEGG_REACTION_ID = "KEGG Reaction ID"
    LEGIOLIST_ID = "LegioList ID"
    LEPROMA_ID = "Leproma ID"
    LOCUS_TAG = "Locus Tag"
    MAIZEGDB_ID = "MaizeGDB ID"
    MEROPS_ID = "MEROPS ID"
    METACYC_REACTION = "MetaCyc Reaction"
    MGC_ZGC_XGC_ID = "MGC(ZGC/XGC) ID"
    MGC_ZGC_XGC_IMAGE_ID = "MGC(ZGC/XGC) Image ID"
    MGC_ZGC_XGC_INFO = "MGC(ZGC/XGC) Info"
    MGI_ID = "MGI ID"
    MIM_ID = "MIM ID"
    MIM_INFO = "MIM Info"
    MINT_ID = "MINT ID"
    MIRBASE_ID = "miRBase ID"
    MIRBASE_MATURE_MIRNA_ACC = "miRBase Mature miRNA Acc"
    MULTIFUN_ID = "MultiFun ID"
    NCIPID_PATHWAY_DESCRIPTION = "NCIPID Pathway Description"
    NCIPID_PATHWAY_NAME = "NCIPID Pathway Name"
    NCIPID_PATHWAY_TITLE = "NCIPID Pathway Title"
    NCIPID_PROTEIN_COMPLEX = "NCIPID Protein Complex"
    NCIPID_PROTEIN_INTERACTOR = "NCIPID Protein Interactor"
    NCIPID_PTM = "NCIPID PTM"
    ORGANISM_SCIENTIFIC_NAME = "Organism Scientific Name"
    ORPHANET_ID = "Orphanet ID"
    PANTHER_ID = "PANTHER ID"
    PARALOG_ENS_GENE_ID = "Paralog - Ens Gene ID"
    PBR_ID = "PBR ID"
    PDB_ID = "PDB ID"
    PEROXIBASE_ID = "PeroxiBase ID"
    PFAM_ID = "Pfam ID"
    PHARMGKB_DISEASE_ID = "PharmGKB Disease ID"
    PHARMGKB_DRUG_ID = "PharmGKB Drug ID"
    PHARMGKB_DRUG_INFO = "PharmGKB Drug Info"
    PHARMGKB_GENE_ID = "PharmGKB Gene ID"
    PIR_ID = "PIR ID"
    PIRSF_ID = "PIRSF ID"
    PPTASEDB_ID = "PptaseDB ID"
    PRINTS_ID = "PRINTS ID"
    PRODOM_ID = "ProDom ID"
    PROSITE_ID = "PROSITE ID"
    PSEUDOCAP_ID = "PseudoCAP ID"
    PUBCHEM_ID = "PubChem ID"
    PUBMED_ID = "PubMed ID"
    REACTOME_ID = "Reactome ID"
    REACTOME_PATHWAY_DESCRIPTION = "Reactome Pathway Description"
    REACTOME_PATHWAY_NAME = "Reactome Pathway Name"
    REBASE_ID = "REBASE ID"
    REFSEQ_GENOMIC_ACCESSION = "RefSeq Genomic Accession"
    REFSEQ_GENOMIC_GI = "RefSeq Genomic GI"
    REFSEQ_MRNA_ACCESSION = "RefSeq mRNA Accession"
    REFSEQ_NCRNA_ACCESSION = "RefSeq ncRNA Accession"
    REFSEQ_NUCLEOTIDE_GI = "RefSeq Nucleotide GI"
    REFSEQ_PROTEIN_ACCESSION = "RefSeq Protein Accession"
    REFSEQ_PROTEIN_GI = "RefSeq Protein GI"
    RFAM_ID = "Rfam ID"
    RGD_ID = "RGD ID"
    SCOP_ID = "SCOP ID"
    SGD_ID = "SGD ID"
    SMART_ID = "SMART ID"
    SNP_INFO = "SNP Info"
    SP_KEYWORDS_ACCESSION = "SP Keywords Accession"
    SP_SUBCELLULAR_LOCATION_ACC = "SP Subcellular Location Acc"
    STRING_PROTEIN_INTERACTOR = "STRING Protein Interactor"
    TAIR_ID = "TAIR ID"
    TAXON_ID = "Taxon ID"
    TCDB_ID = "TCDB ID"
    TIGRFAMS_ID = "TIGRFAMs ID"
    TISSUE_EXPRESSED_IN = "Tissue (expressed in)"
    TUBERCULIST_ID = "TubercuList ID"
    UCSC_ID = "UCSC ID"
    UMBBD_ENZYME_ID = "UM-BBD Enzyme ID"
    UMBBD_PATHWAY_ID = "UM-BBD Pathway ID"
    UNIGENE_ID = "UniGene ID"
    UNIGENE_SEQUENCE_ACCESSION = "UniGene Sequence Accession"
    UNIPROT_ACCESSION = "UniProt Accession"
    UNIPROT_ENTRY_NAME = "UniProt Entry Name"
    UNIPROT_INFO = "UniProt Info"
    UNIPROT_PROTEIN_NAME = "UniProt Protein Name"
    UNISTS_ID = "UniSTS ID"
    UNISTS_INFO = "UniSTS Info"
    VECTORBASE_GENE_ID = "VectorBase Gene ID"
    VEGA_GENE_ID = "VEGA Gene ID"
    VEGA_PROTEIN_ID = "VEGA Protein ID"
    VEGA_TRANSCRIPT_ID = "VEGA Transcript ID"
    WORMBASE_GENE_ID = "WormBase Gene ID"
    WORMPEP_PROTEIN_ID = "WormPep Protein ID"
    XENBASE_GENE_ID = "XenBase Gene ID"
    ZFIN_ID = "ZFIN ID"


if __name__ == "__main__":
    print(Input["GENE_ID"].value)



from dataclasses import dataclass
from typing import List, Literal, Optional, Union

from fast_bioservices.ensembl.ensembl import Ensembl
from fast_bioservices.settings import default_workers


@dataclass
class HomologyTarget:
    protein_id: str
    cigar_line: str
    taxon_id: int
    species: str
    align_seq: str
    id: str
    perc_pos: float
    perc_id: float


@dataclass
class HomologySource:
    align_seq: str
    cigar_line: str
    protein_id: str
    species: str
    taxon_id: int
    perc_pos: float
    perc_id: float
    id: str


@dataclass
class HomologyResult:
    id: str
    method_link_type: str
    taxonomy_level: str
    dn_ds: str
    target: HomologyTarget
    type: str
    source: HomologySource


class GetCafeTree(Ensembl):
    def __init__(self, *, max_workers: int = default_workers, cache: bool = True, show_progress: bool = False):
        super().__init__(max_workers=max_workers, show_progress=show_progress, cache=cache)
        raise NotImplementedError("Not implemented yet")


class GetGeneTree(Ensembl):
    def __init__(self, *, max_workers: int = default_workers, cache: bool = True, show_progress: bool = False):
        super().__init__(max_workers=max_workers, show_progress=show_progress, cache=cache)
        raise NotImplementedError("Not implemented yet")


class GetAlignment(Ensembl):
    def __init__(self, *, max_workers: int = default_workers, cache: bool = True, show_progress: bool = False):
        super().__init__(max_workers=max_workers, show_progress=show_progress, cache=cache)
        raise NotImplementedError("Not implemented yet")


class GetHomology(Ensembl):
    def __init__(
        self,
        max_workers: int = 4,
        show_progress: bool = False,
        cache: bool = True,
    ):
        self._max_workers: int = max_workers
        self._show_progress: bool = show_progress

        super().__init__(
            max_workers=self._max_workers,
            show_progress=self._show_progress,
            cache=cache,
        )

    @property
    def url(self) -> str:
        return self._url

    def by_species_with_symbol_or_id(
        self,
        reference_species: str,
        ensembl_id_or_symbol: Union[str, List[str]],
        aligned: bool = True,
        cigar_line: bool = True,
        compara: str = "vertebrates",
        external_db: str = "",
        format: Literal["full", "condensed"] = "full",
        sequence: Literal["none", "cdna", "protein"] = "protein",
        target_species: str = "",
        target_taxon: Optional[int] = None,
        type: Literal["orthologues", "paralogues", "projections", "all"] = "all",
    ) -> List[HomologyResult]:
        ensembl_id_or_symbol = [ensembl_id_or_symbol] if isinstance(ensembl_id_or_symbol, str) else ensembl_id_or_symbol

        urls = []
        for e_id in ensembl_id_or_symbol:
            path = f"/homology/symbol/{reference_species}/{e_id}?"
            if e_id.startswith("ENSG"):
                path = path.replace("/symbol/", "/id/")
            path += f"compara={compara};format={format};sequence={sequence};type={type}"

            if aligned:
                path += ";aligned"
            if cigar_line:
                path += ";cigar_line"
            if external_db != "":
                path += f";external_db={external_db}"
            if target_species != "":
                path += f";target_species={target_species}"
            if target_taxon is not None:
                path += f";target_taxon={target_taxon}"
            urls.append(self._url + path)

        homology_results: list[HomologyResult] = []
        results = self._get(
            urls=urls,
            headers={"Content-Type": "application/json"},
        )
        for result in results:
            id_ = result.json["data"][0]["id"]
            homologies: list = result.json["data"][0]["homologies"]
            for homology in homologies:
                homology_results.append(HomologyResult(**homology, id=id_))
        return homology_results


def main():
    e = GetHomology(max_workers=1, show_progress=True)
    e.by_species_with_symbol_or_id(
        reference_species="human",
        ensembl_id_or_symbol="ENSG00000157764",
        target_species="macaca_mulatta",
    )


if __name__ == "__main__":
    main()



from fast_bioservices.ensembl.comparative_genomics import (
    GetAlignment,
    GetCafeTree,
    GetGeneTree,
    GetHomology,
    HomologyResult,
)
from fast_bioservices.ensembl.cross_references import CrossReference

__all__ = ["GetAlignment", "GetCafeTree", "GetGeneTree", "GetHomology", "HomologyResult", "CrossReference"]



__all__ = ["BiGG", "BioDBNet", "Input", "Output", "Taxon"]
__version__ = "0.3.4"
__description__ = "A fast way to access and convert biological information"

from fast_bioservices.bigg.bigg import BiGG
from fast_bioservices.biodbnet.biodbnet import BioDBNet
from fast_bioservices.biodbnet.nodes import Input, Output, Taxon



from dataclasses import dataclass
from typing import List, Literal, Optional, Union

from fast_bioservices.ensembl.ensembl import Ensembl, Species
from fast_bioservices.settings import default_workers


@dataclass(frozen=True)
class EnsemblReference:
    input: str
    type: str
    id: str


@dataclass(frozen=True)
class ExternalReference:
    description: str
    info_text: str
    synonyms: List[str]
    dbname: str
    info_type: str
    db_display_name: str
    display_id: str
    version: str
    primary_id: str

    def __post_init__(self):
        if object.__getattribute__(self, "description") is None:
            object.__setattr__(self, "description", "")


class CrossReference(Ensembl):
    def __init__(
        self,
        max_workers: int = default_workers,
        show_progress: bool = False,
        cache: bool = True,
    ):
        self._max_workers: int = max_workers
        self._show_progress: bool = show_progress

        super().__init__(
            max_workers=self._max_workers,
            show_progress=self._show_progress,
            cache=cache,
        )

    def get_ensembl_from_external(
        self,
        species: str,
        gene_symbols: Union[str, List[str]],
        db_type: Literal["core"] = "core",
        external_db_filter: Optional[str] = None,
        feature_filter: Optional[str] = None,
    ):
        validate_species: Optional[Species] = self._match_species(species)
        assert validate_species is not None, f"Species {species} not found"

        gene_symbols = [gene_symbols] if isinstance(gene_symbols, str) else gene_symbols

        urls = []
        for symbol in gene_symbols:
            path = f"/xrefs/symbol/{validate_species.common_name}/{symbol}?db_type={db_type}"
            if external_db_filter:
                path += f";external_db={external_db_filter}"
            if feature_filter:
                path += f";object_type={feature_filter}"
            urls.append(self._url + path)

        references: list[EnsemblReference] = []
        for i, result in enumerate(self._get(urls=urls, headers={"Content-Type": "application/json"})):
            as_json = result.json[0]
            references.append(EnsemblReference(**as_json, input=gene_symbols[i]))
        return references

    def get_external_from_ensembl(
        self,
        ensembl_id: Union[str, List[str]],
        db_type: Literal["core"] = "core",
        all_levels: bool = False,
        external_db_filter: Optional[str] = None,
        feature_filter: Optional[str] = None,
        species: Optional[str] = None,
    ) -> List[ExternalReference]:
        ensembl_id = [ensembl_id] if isinstance(ensembl_id, str) else ensembl_id

        urls = []
        for e_id in ensembl_id:
            path = f"/xrefs/id/{e_id}?db_type={db_type}"
            if all_levels:
                path += ";all_levels=1"
            if external_db_filter:
                path += f";external_db={external_db_filter}"
            if feature_filter:
                path += f";object_type={feature_filter}"
            if species:
                path += f";species={species}"
            urls.append(self._url + path)

        references: list[ExternalReference] = []
        for result in self._get(urls=urls, headers={"Content-Type": "application/json"}):
            as_json = result.json[0]
            references.append(ExternalReference(**as_json))

        return references

    @property
    def url(self) -> str:
        return self._url


def main():
    c = CrossReference(max_workers=1)

    r = c.get_ensembl_from_external("human", ["GOLT1A", "GOLT1B"])
    print(r)


if __name__ == "__main__":
    main()



from pathlib import Path

import appdirs

# Cache settings
_root_cache_dir: Path = Path(appdirs.user_cache_dir("fast_bioservices"))
cache_dir: Path = Path(_root_cache_dir, "fast_bioservices_cache")
log_filepath: Path = Path(_root_cache_dir, "fast_bioservices.log")

_root_cache_dir.mkdir(parents=True, exist_ok=True)
cache_dir.mkdir(parents=True, exist_ok=True)
log_filepath.touch()
default_workers: int = 4



import os

import pytest
from fast_bioservices import BiGG


@pytest.fixture(scope="session")
def bigg_instance():
    return BiGG(max_workers=1, cache=False, show_progress=False)


def test_version(bigg_instance):
    current_version = bigg_instance.version()
    assert current_version["api_version"] == "v2"
    assert "bigg_models_version" in current_version.keys()


def test_models(bigg_instance):
    models = bigg_instance.models()
    assert len(models.keys()) > 0
    assert len(models["results"]) > 0
    assert "results_count" in models.keys()


def test_model_details(bigg_instance):
    recon_3d = bigg_instance.model_details("Recon3D")
    keys = ["metabolite_count", "reaction_count", "gene_count", "genome_name", "json_size", "xml_size", "mat_size", "json_gz_size", "xml_gz_size", "mat_gz_size", "organism", "genome_ref_string", "reference_type", "reference_id", "model_bigg_id", "published_filename"]  # fmt: skip
    values = [5835, 10600, 2248, "GCF_000001405.33", "7.5 MB", "27.2 MB", "477.2 MB", "932.1 kB", "1.2 MB", "983.6 kB", "Homo sapiens", "ncbi_assembly:GCF_000001405.33", "pmid", "29457794", "Recon3D", "Recon3D.mat"]  # fmt: skip

    for key, value in zip(keys, values):
        assert recon_3d[key] == value

    assert "escher_maps" in recon_3d.keys()
    assert "last_updated" in recon_3d.keys()


def test_json(bigg_instance):
    assert len(bigg_instance.json("Recon3D").keys()) > 0


def test_download(bigg_instance):
    download_path = os.getcwd()
    bigg_instance.download("Recon3D", format="json.gz", download_path=download_path)
    assert "Recon3D.json.gz" in list(os.listdir())
    os.unlink(f"{download_path}/Recon3D.json.gz")


def test_model_reactions(bigg_instance):
    reactions = bigg_instance.model_reactions("Recon3D")
    assert len(reactions.keys()) > 0
    assert "results_count" in reactions.keys()


def test_model_reaction_details(bigg_instance):
    reaction_details = bigg_instance.model_reaction_details("Recon3D", "HEX1")

    general_keys = {"results", "database_links", "escher_maps", "old_identifiers", "bigg_id", "model_bigg_id", "count", "pseudoreaction", "name", "other_models_with_reaction", "metabolites"}  # fmt: skip
    results_keys = {"exported_reaction_id", "copy_number", "gene_reaction_rule", "upper_bound", "genes", "lower_bound", "objective_coefficient", "subsystem", "reaction_string"}  # fmt: skip
    database_keys = {"RHEA", "KEGG Reaction", "MetaNetX (MNX) Equation", "BioCyc", "EC Number", "SEED Reaction"}  # fmt: skip

    # Check if all elements on right are a part of the left
    assert set(reaction_details.keys()) >= general_keys
    assert set(reaction_details["results"][0].keys()) >= results_keys
    assert set(reaction_details["database_links"].keys()) >= database_keys


def test_model_metabolites(bigg_instance): ...
def test_model_metabolite_details(bigg_instance): ...
def test_model_genes(bigg_instance): ...
def test_model_gene_details(bigg_instance): ...
def test_universal_reactions(bigg_instance): ...
def test_universal_reaction_details(bigg_instance): ...
def test_universal_metabolites(bigg_instance): ...
def test_universal_metabolite_details(bigg_instance): ...
def test_search(bigg_instance): ...



import pytest
from fast_bioservices.ensembl.cross_references import CrossReference, EnsemblReference, ExternalReference


@pytest.fixture(scope="session")
def cross_reference() -> CrossReference:
    return CrossReference(max_workers=1, cache=False, show_progress=False)


def test_get_ensembl_from_external(cross_reference: CrossReference):
    expected = EnsemblReference(input="BRCA1", type="gene", id="ENSG00000012048")
    actual = cross_reference.get_ensembl_from_external(species="humans", gene_symbols="BRCA1")[0]

    assert expected.input == actual.input
    assert expected.type == actual.type
    assert expected.id == actual.id


def test_get_external_from_ensembl(cross_reference: CrossReference):
    expected = ExternalReference(
        description="Locus Reference Genomic record for BRCA1",
        info_text="",
        synonyms=[],
        dbname="ENS_LRG_gene",
        info_type="DIRECT",
        db_display_name="LRG display in Ensembl gene",
        display_id="LRG_292",
        version="0",
        primary_id="LRG_292",
    )
    actual = cross_reference.get_external_from_ensembl(ensembl_id="ENSG00000012048")[0]

    assert expected.description == actual.description
    assert expected.info_text == actual.info_text
    assert expected.synonyms == actual.synonyms
    assert expected.dbname == actual.dbname
    assert expected.info_type == actual.info_type
    assert expected.db_display_name == actual.db_display_name
    assert expected.display_id == actual.display_id
    assert expected.version == actual.version
    assert expected.primary_id == actual.primary_id
